
provider "oci" {
  tenancy_ocid     = "ocid1.tenancy.oc1..aaaaaaaarod5yc3653ujwgvjbqui3s6r6ntfbs3d4uwxlkitku5flcbktnga"
  user_ocid        = "ocid1.user.oc1..aaaaaaaa5jnxktye7itiq4fvgkm5ftznniph7ncwlwhp6rzvxft4l23ay2ea"
  fingerprint      = "ac:14:bf:a5:47:a2:47:e1:59:e5:f9:2f:47:42:1e:f5"
  private_key_path = "/home/iqbalbadus/.oci/oci_api_key.pem"
  region           = "ap-hyderabad-1"
}

resource "oci_core_vcn" "internal_vcn" {
  count = var.internal_existing_vcn ? 0 : 1
  cidr_block     = var.internal_vcn_cidr_block
  display_name   = var.internal_vcn_display_name
  compartment_id = var.compartment_id
}

locals {
  internal_vcn_id = var.internal_existing_vcn ? var.internal_vcn_id : oci_core_vcn.internal_vcn[0].id
}

resource "oci_core_vcn" "external_vcn" {
  count = var.external_existing_vcn ? 0 : 1
  cidr_block     = var.external_vcn_cidr_block
  display_name   = var.external_vcn_display_name
  compartment_id = var.compartment_id
}

locals {
  ext_tmp_vcn_id = var.external_existing_vcn ? var.external_vcn_id : oci_core_vcn.external_vcn[0].id
  external_vcn_id = var.use_internal_vcn_for_all ? local.internal_vcn_id : local.ext_tmp_vcn_id
}

resource "oci_core_internet_gateway" "ext_internet_gateway" {
  count = var.create_external_ig ? 1 : 0
  vcn_id         = local.external_vcn_id
  display_name   = var.external_ig_name
  compartment_id = var.compartment_id
}

resource "oci_core_route_table" "external_rt" {
  count = var.create_external_ig ? 1 : 0
  compartment_id = var.compartment_id
  vcn_id         = local.external_vcn_id
  display_name   = var.external_rt_name

  route_rules {
    destination       = "0.0.0.0/0"
    destination_type  = "CIDR_BLOCK"
    network_entity_id = oci_core_internet_gateway.ext_internet_gateway[0].id
  }
}

resource "oci_core_subnet" "external_subnet" {
  count = var.external_existing_subnet ? 0 : 1
  vcn_id         = local.external_vcn_id
  display_name   = var.external_subnet_display_name
  cidr_block     = var.external_subnet_cidr_block
  compartment_id = var.compartment_id
  prohibit_public_ip_on_vnic = false
  route_table_id = var.create_external_ig ? oci_core_route_table.external_rt[0].id : null
}

locals {
  external_subnet_id = var.external_existing_subnet ? var.external_subnet_id : oci_core_subnet.external_subnet[0].id
}


resource "oci_core_vcn" "management_vcn" {
  count = var.enable_management? (var.management_existing_vcn ? 0 : 1) : 0
  cidr_block     = var.management_vcn_cidr_block
  display_name   = var.management_vcn_display_name
  compartment_id = var.compartment_id
}

locals {
  mgt_tmp_vcn_id = var.enable_management? (var.management_existing_vcn ? var.management_vcn_id : oci_core_vcn.management_vcn[0].id) : null
  management_vcn_id = var.enable_management? ( var.use_internal_vcn_for_all ? local.internal_vcn_id : local.mgt_tmp_vcn_id) : null
}


resource "oci_core_nat_gateway" "management_nat_gateway" {
  count = var.enable_management? (var.create_management_nat ? 1 : 0) : 0
  vcn_id         = local.management_vcn_id
  display_name   = var.management_nat_name
  compartment_id = var.compartment_id
}

resource "oci_core_route_table" "management_rt" {
  count = var.enable_management? (var.create_management_nat ? 1 : 0) : 0
  compartment_id = var.compartment_id
  vcn_id         = local.management_vcn_id
  display_name   = var.management_rt_name
  route_rules {
    destination        = "0.0.0.0/0"
    destination_type  = "CIDR_BLOCK"
    network_entity_id = oci_core_nat_gateway.management_nat_gateway[0].id
  }
}


resource "oci_core_subnet" "management_subnet" {
  count = var.enable_management? (var.management_existing_subnet ? 0 : 1) : 0
  vcn_id         = local.management_vcn_id
  display_name   = var.management_subnet_display_name
  cidr_block     = var.management_subnet_cidr_block
  compartment_id = var.compartment_id
  route_table_id = var.create_management_nat ? oci_core_route_table.management_rt[0].id : null
}

locals {
  management_subnet_id = var.enable_management? (var.management_existing_subnet ? var.management_subnet_id : oci_core_subnet.management_subnet[0].id) : null
}

resource "oci_core_nat_gateway" "internal_nat_gateway" {
  count = var.create_internal_nat ? 1 : 0
  vcn_id         = local.internal_vcn_id
  display_name   =  var.internal_nat_name
  compartment_id = var.compartment_id
}

resource "oci_core_route_table" "internal_rt" {
  count = var.create_internal_nat ? 1 : 0
  compartment_id = var.compartment_id
  vcn_id         = local.internal_vcn_id
  display_name   =  var.internal_rt_name
  route_rules {
    destination        = "0.0.0.0/0"
    destination_type  = "CIDR_BLOCK"
    network_entity_id = oci_core_nat_gateway.internal_nat_gateway[0].id
  }
}

resource "oci_core_subnet" "internal_subnet" {
  count = var.internal_existing_subnet ? 0 : 1
  vcn_id         = local.internal_vcn_id
  display_name   = var.internal_subnet_display_name
  cidr_block     = var.internal_subnet_cidr_block
  compartment_id = var.compartment_id
  route_table_id = var.create_internal_nat ? oci_core_route_table.internal_rt[0].id : null
}

locals {
  internal_subnet_id = var.internal_existing_subnet ? var.internal_subnet_id : oci_core_subnet.internal_subnet[0].id
}

resource "oci_core_network_security_group" "internal_nsg" {
  compartment_id = var.compartment_id
  vcn_id         = local.internal_vcn_id
  display_name   = var.internal_nsg_name
}

resource "oci_core_network_security_group_security_rule" "internal_nsg_rules" {
  for_each = { for rule in var.internal_nsg_rules : "${rule.direction}-${rule.protocol}-${rule.source}-${rule.min_dstport}-${rule.min_srcport}-${rule.max_dstport}-${rule.max_srcport}-${rule.destination}-${rule.description}" => rule }

  network_security_group_id = oci_core_network_security_group.internal_nsg.id

  direction = each.value.direction
  protocol  = each.value.protocol

 dynamic "udp_options" {
      for_each = each.value.protocol == "17" ? [1] : []
      content {
        dynamic "destination_port_range" {
          for_each = each.value.max_dstport > 0 ? [{ min = each.value.min_dstport, max = each.value.max_dstport }] : []
          content {
            min = destination_port_range.value.min
            max = destination_port_range.value.max
          }
        }
        dynamic "source_port_range" {
          for_each = each.value.max_srcport > 0 ? [{ min = each.value.min_srcport, max = each.value.max_srcport }] : []
          content {
            min = source_port_range.value.min
            max = source_port_range.value.max
          }
        }
      }
  }
  dynamic "tcp_options" {
        for_each = each.value.protocol == "6" ? [1] : []
    content {
        dynamic "destination_port_range" {
          for_each = each.value.max_dstport > 0 ? [{ min = each.value.min_dstport, max = each.value.max_dstport }] : []
          content {
            min = destination_port_range.value.min
            max = destination_port_range.value.max
          }
        }
        dynamic "source_port_range" {
          for_each = each.value.max_srcport > 0 ? [{ min = each.value.min_srcport, max = each.value.max_srcport }] : []
          content {
            min = source_port_range.value.min
            max = source_port_range.value.max
          }
        }
    }
  }
  
  destination = each.value.destination
  source = each.value.source
}

resource "oci_core_network_security_group" "external_nsg" {
  compartment_id = var.compartment_id
  vcn_id         = local.external_vcn_id
  display_name   =  var.external_nsg_name
}

resource "oci_core_network_security_group_security_rule" "external_nsg_rules" {
  for_each = { for rule in var.external_nsg_rules : "${rule.direction}-${rule.protocol}-${rule.source}-${rule.min_dstport}-${rule.min_srcport}-${rule.max_dstport}-${rule.max_srcport}-${rule.destination}-${rule.description}" => rule }

  network_security_group_id = oci_core_network_security_group.external_nsg.id

  direction = each.value.direction
  protocol  = each.value.protocol
  dynamic "udp_options" {
      for_each = each.value.protocol == "17" ? [1] : []
      content {
        dynamic "destination_port_range" {
          for_each = each.value.max_dstport > 0 ? [{ min = each.value.min_dstport, max = each.value.max_dstport }] : []
          content {
            min = destination_port_range.value.min
            max = destination_port_range.value.max
          }
        }
        dynamic "source_port_range" {
          for_each = each.value.max_srcport > 0 ? [{ min = each.value.min_srcport, max = each.value.max_srcport }] : []
          content {
            min = source_port_range.value.min
            max = source_port_range.value.max
          }
        }
      }
  }
  dynamic "tcp_options" {
        for_each = each.value.protocol == "6" ? [1] : []
    content {
        dynamic "destination_port_range" {
          for_each = each.value.max_dstport > 0 ? [{ min = each.value.min_dstport, max = each.value.max_dstport }] : []
          content {
            min = destination_port_range.value.min
            max = destination_port_range.value.max
          }
        }
        dynamic "source_port_range" {
          for_each = each.value.max_srcport > 0 ? [{ min = each.value.min_srcport, max = each.value.max_srcport }] : []
          content {
            min = source_port_range.value.min
            max = source_port_range.value.max
          }
        }
    }
  }
  destination = each.value.destination
  source = each.value.source
}


resource "oci_core_network_security_group" "management_nsg" {
  count = var.enable_management? 1 : 0
  compartment_id = var.compartment_id
  vcn_id         = local.management_vcn_id
  display_name   =  var.management_nsg_name
}

resource "oci_core_network_security_group_security_rule" "management_nsg_rules" {
  for_each = var.enable_management ? ({ for rule in var.management_nsg_rules : "${rule.direction}-${rule.protocol}-${rule.source}-${rule.min_dstport}-${rule.min_srcport}-${rule.max_dstport}-${rule.max_srcport}-${rule.destination}-${rule.description}" => rule }) : {}

  network_security_group_id = var.enable_management? (oci_core_network_security_group.management_nsg[0].id) : 0

  direction = each.value.direction
  protocol  = each.value.protocol

  dynamic "udp_options" {
      for_each = each.value.protocol == "17" ? [1] : []
      content {
        dynamic "destination_port_range" {
          for_each = each.value.max_dstport > 0 ? [{ min = each.value.min_dstport, max = each.value.max_dstport }] : []
          content {
            min = destination_port_range.value.min
            max = destination_port_range.value.max
          }
        }
        dynamic "source_port_range" {
          for_each = each.value.max_srcport > 0 ? [{ min = each.value.min_srcport, max = each.value.max_srcport }] : []
          content {
            min = source_port_range.value.min
            max = source_port_range.value.max
          }
        }
      }
  }
  dynamic "tcp_options" {
        for_each = each.value.protocol == "6" ? [1] : []
    content {
        dynamic "destination_port_range" {
          for_each = each.value.max_dstport > 0 ? [{ min = each.value.min_dstport, max = each.value.max_dstport }] : []
          content {
            min = destination_port_range.value.min
            max = destination_port_range.value.max
          }
        }
        dynamic "source_port_range" {
          for_each = each.value.max_srcport > 0 ? [{ min = each.value.min_srcport, max = each.value.max_srcport }] : []
          content {
            min = source_port_range.value.min
            max = source_port_range.value.max
          }
        }
    }
  }

  destination = each.value.destination
  source = each.value.source
}

locals {
  metadata = {
    pulse-config = var.init_config
  }
}

resource "oci_core_instance" "my_instance" {
  availability_domain = var.availability_domain
  compartment_id      = var.compartment_id
  shape               = var.shape
  shape_config {
    ocpus             = var.total_flex_OCPUs
    memory_in_gbs     = var.total_flex_RAM
  }
  display_name        = var.vm_display_name
  source_details {
    source_type = "image"
    source_id = var.image_id
  } 
  metadata 	      = local.metadata

  create_vnic_details {
    subnet_id        = local.internal_subnet_id
    display_name     = var.internal_nic_display_name
    assign_public_ip = false
    nsg_ids          = [oci_core_network_security_group.internal_nsg.id] # Associate the NSG here
    private_ip =  var.use_static_internal_ip ? var.internal_ip_address : null

  }

}

resource "oci_core_vnic_attachment" "external_vnic_attachment" {
  instance_id    = oci_core_instance.my_instance.id
  display_name   = var.external_nic_display_name
  
  create_vnic_details {
    subnet_id        = local.external_subnet_id
    display_name     = var.external_nic_display_name
    assign_public_ip = true
    nsg_ids          = [oci_core_network_security_group.external_nsg.id] # Associate the NSG here
    private_ip =  var.use_static_external_ip ? var.external_ip_address : null

  }

  provisioner "local-exec" {
  	command = "python ./nzta_update_ip.py -v ${oci_core_vnic_attachment.external_vnic_attachment.vnic_id} -i ${oci_core_instance.my_instance.id}"
  }
}


data "oci_core_private_ips" "external_vnic_ips" {
  vnic_id = oci_core_vnic_attachment.external_vnic_attachment.vnic_id
}

resource "oci_core_vnic_attachment" "management_vnic_attachment" {
  count = var.enable_management?  1 : 0
  instance_id    = oci_core_instance.my_instance.id
  display_name   = var.management_nic_display_name

  depends_on = [oci_core_vnic_attachment.external_vnic_attachment]

  create_vnic_details {
    subnet_id        = local.management_subnet_id
    display_name     = var.management_nic_display_name
    assign_public_ip = false
    nsg_ids          = [oci_core_network_security_group.management_nsg[0].id] # Associate the NSG here
    private_ip =  var.use_static_management_ip ? var.management_ip_address : null

  }
}


resource "oci_network_load_balancer_network_load_balancer" "external_nlb" {
 
  count = var.use_load_balancer ? (var.use_existing_lb_for_ext_nic  ? 0 : 1) : 0
  compartment_id = var.compartment_id
  display_name   = var.lb_display_name
  is_preserve_source_destination = false
  subnet_id      =  local.external_subnet_id
  network_security_group_ids = [oci_core_network_security_group.external_nsg.id]
  is_private = false

}

locals {
  ext_lb_id = var.use_load_balancer ? (var.use_existing_lb_for_ext_nic ? var.external_lb_id : oci_network_load_balancer_network_load_balancer.external_nlb[0].id) : null
}

resource "oci_network_load_balancer_backend_set" "external_backend_set" { 
  count = var.use_load_balancer ? (var.use_existing_bs_for_ext_nic  ? 0 : length(var.ports)) : 0
  network_load_balancer_id = local.ext_lb_id
  name                     = format("%s", var.bs_display_name[count.index])
  policy                   = var.bs_policy

  health_checker {
    port     = 443
    protocol = "TCP"
  }

}


resource "oci_network_load_balancer_listener" "external_listener" {
  
  count = var.use_load_balancer ? (var.use_existing_ls_for_ext_nic  ? 0 : length(var.ports)) : 0

  depends_on = [oci_network_load_balancer_backend_set.external_backend_set]

  network_load_balancer_id = local.ext_lb_id
  name                     = format("%s", var.listener_display_name[count.index])
  default_backend_set_name = format("%s", var.bs_display_name[count.index])
  port                     = var.ports[count.index]
  protocol                 = var.protocol
  
}

resource "oci_network_load_balancer_backend" "external_backend" {
  count               = var.use_load_balancer ? length(var.ports) : 0

  network_load_balancer_id = local.ext_lb_id
  backend_set_name         = format("%s",var.bs_display_name[count.index] )
  ip_address               = data.oci_core_private_ips.external_vnic_ips.private_ips[0].ip_address
  port                     = var.ports[count.index]
  weight                   = 1
}