// -------------------
// Global Variables
// -------------------

variable "enable_management" {
  description = "Set it to True ,if nZTA needs an add additional nic for management interface"
  type        = bool
  default     = true
}

variable "internal_existing_vcn" {
  description = "Set it to True, if new VCN needs to be created for internal(primary) interface."
  type        = bool
  default     = true
}

variable "external_existing_vcn" {
  description = "Set it to True, if new VCN needs to be created for external interface."
  type        = bool
  default     = true
}

variable "management_existing_vcn" {
	description = "Set it to True, if new VCN needs to be created for management interface."
	type        = bool
	default     = true
}

variable "use_internal_vcn_for_all" {
	description = "Set it to True, if internal VCN needs to be used for all other interfaces."
	type        = bool
	default     = true
}

variable "use_load_balancer" {
	description = "Set it to True, if network load balancer needs to be assigned."
	type        = bool
	default     = true
}

variable "use_existing_lb_for_ext_nic" {
	description = "Set it to True, if existing load balancer to be used."
	type        = bool
	default     = true
}

variable "use_existing_bs_for_ext_nic" {
	description = "Set it to True, if existing backend set to be used."
	type        = bool
	default     = true
}

variable "use_existing_ls_for_ext_nic" {
	description = "Set it to True, if existing listener to be used."
	type        = bool
	default     = true
}

variable "internal_existing_subnet" {
	description = "Set it to True, if existing subnet to be used."
	type        = bool
	default     = true
}

variable "external_existing_subnet" {
	description = "Set it to True, if existing subnet to be used."
	type        = bool
	default     = true
}

variable "management_existing_subnet" {
	description = "Set it to True, if existing subnet to be used."
	type        = bool
	default     = true
}

variable "create_management_nat" {
	description = "Set it to True, if management nat needs to be created."
	type        = string
	default     = "my_management_nat"
}

variable "create_internal_nat" {
	description = "Set it to True, if internal nat needs to be created."
	type        = string
	default     = "my_internal_nat"
}

variable "create_external_ig" {
	description = "Set it to True, if exteranl internet gateway needs to be created."
	type        = string
	default     = "my_external_ig"
}

variable "use_static_management_ip" {
	description = "Set it to True, if static IP to used for for management nic."
	type        = bool
	default     = false
}

variable "use_static_external_ip" {
	description = "Set it to True, if static IP to used for for external nic."
	type        = bool
	default     = false
}

variable "use_static_internal_ip" {
	description = "Set it to True, if static IP to used for for internal nic."
	type        = bool
	default     = false
}


// -------------------
// Compartment Variables
// -------------------

variable "compartment_id" {
	description = "The OCID of the compartment where the nZTA gateway will be deployed"
	type        = string
	default     = "my_compartment_ocid"
}

// -------------------
// VM Variables
// -------------------

variable "vm_display_name" {
	description = "Display name for the nZTA gateway VM."
	type        = string
	default     = "my_vm"
}

variable "image_id" {
	description = "OCID of the image to use for the nZTA gateway deployment."
	type        = string
	default     = "my_vm_image"
}

variable "shape" {
	description = "VM shape for the nZTA gateway (e.g., VM.Standard.E4.Flex and etc.)"
	type        = string
	default     = "VM.Standard.E4.Flex"
}

variable "total_flex_OCPUs" {
	description = "Number of required OCPU's for the flex VM"
	type        = number
	default     = 3
}

variable "total_flex_RAM" {
	description = "RAM size(GB) for the flex VM."
	type        = number
	default     = 16
}

variable "availability_domain" {
	description = "Availability domain where the shape belong to"
	type        = string
	default     = "my-availability-domain"
}


// -------------------
// Internal VCN Info
// -------------------

variable "internal_vcn_cidr_block" {
	description = "CIDR block for the Virtual Cloud Network"
	type        = string
	default     = "10.0.0.0/16"
}

variable "internal_vcn_display_name" {
	description = "Display name for the Virtual Cloud Network"
	type        = string
	default     = "my_internal_vcn"
}

// -------------------
// Internal Subnet Info
// -------------------

variable "internal_vcn_id" {
	description = "OCID of the vcn to use for the nZTA internal interface (Primay VNIC)"
	type        = string
	default     = "my_internal_vcn_id"
}

variable "internal_subnet_id" {
	description = "OCID of the subnet to use for the nZTA internal interface (Primay VNIC). "
	type        = string
	default     = "my_internal_subnet_id"
}

variable "internal_subnet_cidr_block" {
	description = "CIDR block for the internal subnet.It is required only if the internal_existing_vcn is false. "
	type        = string
	default     = "10.0.0.0/18"
}

variable "internal_subnet_display_name" {
	description = "Display name for the internal subnet.It is required  only if the internal_existing_vcn is false."
	type        = string
	default     = "my_internal_subnet"
}

variable "internal_rt_name" {
	description = "Display name for the internal route table."
	type        = string
	default     = "internal_rt_name"
}

variable "internal_nat_name" {
	description = "Display name for the internal route gateway."
	type        = string
	default     = "internal_nat_name"
}



// -------------------
// Internal Nic Info
// -------------------

variable "internal_nic_display_name" {
	description = "Display name for the internal NIC (Primary VNIC)"
	type        = string
	default     = "my_internal_nic"
}

variable "internal_ip_address" {
	description = "Static IP address of the internal nic (optional) "
	type        = string
	default     = "10.0.0.2"
}

variable "internal_is_public_ip_enabled" {
	description = "Assign public IP to internal nic."
	type        = bool
	default     = false
}

variable "internal_nsg_name" {
	description = "Display name for the internal nsg group."
	type        = string
	default     = "internal_nsg_name"
}

variable "internal_nsg_rules" {
	description = "List of NSG rules"
	type = list(object({
	  protocol   	= string
	  destination   = string
	  source     	= string
	  min_dstport   = number
	  min_srcport	= number
	  max_dstport   = number
	  max_srcport	= number
	  description   = string
	  direction		= string
	}))
  
	default = [
		{
			protocol = "6" # TCP
			destination   = "0.0.0.0/0"
			source   = "0.0.0.0/0"
			min_dstport   = 22
			min_srcport	= 22
			max_dstport   = 22
			max_srcport	= 22
			description = "internal ingress"
			direction = "INGRESS"

		},
		{
			protocol = "6" # TCP
			destination   = "0.0.0.0/0"
			source   = "0.0.0.0/0"
			min_dstport   = 80
			min_srcport	= 80
			max_dstport   = 80
			max_srcport	= 80
			description = "internal ingress"
			direction = "EGRESS"
		}
	]
  }

// -------------------
// external VCN Info
// -------------------

variable "external_vcn_cidr_block" {
	description = "CIDR block for the Virtual Cloud Network"
	type        = string
	default     = "10.0.0.0/16"
}

variable "external_vcn_display_name" {
	description = "Display name for the Virtual Cloud Network"
	type        = string
	default     = "my_external_vcn"
}

// -------------------
// external Subnet Info
// -------------------

variable "external_vcn_id" {
	description = "OCID of the vcn to use for the nZTA external interface"
	type        = string
	default     = "my_external_vcn_id"
}

variable "external_subnet_id" {
	description = "OCID of the subnet to use for the nZTA external interface."
	type        = string
	default     = "my_external_subnet_id"
}

variable "external_subnet_cidr_block" {
	description = "CIDR block for the external subnet.It is required only if the external_existing_vcn is false. "
	type        = string
	default     = "10.0.0.0/18"
}

variable "external_subnet_display_name" {
	description = "Display name for the external subnet.It is required  only if the external_existing_vcn is false."
	type        = string
	default     = "my_external_subnet"
}

variable "external_rt_name" {
	description = "Display name for the external route table."
	type        = string
	default     = "external_rt_name"
}

variable "external_ig_name" {
	description = "Display name for the external internet gateway."
	type        = string
	default     = "external_ig_name"
}

variable "external_nsg_name" {
	description = "Display name for the external nsg group."
	type        = string
	default     = "external_nsg_name"
}


// -------------------
// external Nic Info
// -------------------

variable "external_nic_display_name" {
	description = "Display name for the external NIC."
	type        = string
	default     = "my_external_nic"
}

variable "external_ip_address" {
	description = "Static IP address of the external nic (optional) "
	type        = string
	default     = "10.0.0.2"
}

variable "external_is_public_ip_enabled" {
	description = "Assign public IP to external nic."
	type        = bool
	default     = false
}

variable "external_nsg_rules" {
	description = "List of ingress rules"
	type = list(object({
	  protocol   	= string
	  destination 	= string
	  source     	= string
	  min_dstport   = number
	  min_srcport	= number
	  max_dstport   = number
	  max_srcport	= number
	  description   = string
	  direction		= string
	}))
  
	default = [
		{
			protocol = "6" # TCP
			destination  = "0.0.0.0/0"
			source   = "0.0.0.0/0"
			min_dstport   = 22
			min_srcport	= 22
			max_dstport   = 22
			max_srcport	= 22
			description = "external ingress"
			direction = "INGRESS"
		},
		{
			protocol = "6" # TCP
			source   = "0.0.0.0/0"
			destination   = "0.0.0.0/0"
			min_dstport   = 80
			min_srcport	= 80
			max_dstport   = 80
			max_srcport	= 80
			description = "external ingress"
			direction = "EGRESS"
		}
	]
  }

// -------------------
// management VCN Info
// -------------------

variable "management_vcn_cidr_block" {
	description = "CIDR block for the Virtual Cloud Network"
	type        = string
	default     = "10.0.0.0/16"
}

variable "management_vcn_display_name" {
	description = "Display name for the Virtual Cloud Network"
	type        = string
	default     = "my_management_vcn"
}

// -------------------
// management Subnet Info
// -------------------

variable "management_vcn_id" {
	description = "OCID of the vcn to use for the nZTA management interface"
	type        = string
	default     = "my_management_vcn_id"
}

variable "management_subnet_id" {
	description = "OCID of the subnet to use for the nZTA management interface."
	type        = string
	default     = "my_management_subnet_id"
}

variable "management_subnet_cidr_block" {
	description = "CIDR block for the management subnet.It is required only if the management_existing_vcn is false. "
	type        = string
	default     = "10.0.0.0/18"
}

variable "management_subnet_display_name" {
	description = "Display name for the management subnet.It is required  only if the management_existing_vcn is false."
	type        = string
	default     = "my_management_subnet"
}

variable "management_rt_name" {
	description = "Display name for the management route table."
	type        = string
	default     = "internal_rt_name"
}

variable "management_nat_name" {
	description = "Display name for the management nat gateway."
	type        = string
	default     = "internal_nat_name"
}

// -------------------
// management Nic Info
// -------------------

variable "management_nic_display_name" {
	description = "Display name for the management NIC."
	type        = string
	default     = "my_management_nic"
}

variable "management_ip_address" {
	description = "Static IP address of the management nic (optional) "
	type        = string
	default     = "10.0.0.2"
}

variable "management_is_public_ip_enabled" {
	description = "Assign public IP to management nic."
	type        = bool
	default     = false
}

variable "management_nsg_name" {
	description = "Display name for the management nsg group."
	type        = string
	default     = "management_nsg_name"
}

variable "management_nsg_rules" {
	description = "List of ingress rules"
	type = list(object({
	  protocol   	= string
	  destination	= string
	  source     	= string
	  min_dstport   = number
	  min_srcport	= number
	  max_dstport   = number
	  max_srcport	= number
	  description   = string
	  direction		= string
	}))
  
	default = [
	  {
		protocol = "6" # TCP
		destination   = "0.0.0.0/0"
		source   = "0.0.0.0/0"
		min_dstport   = 22
		min_srcport	= 22
		max_dstport   = 22
		max_srcport	= 22
		description = "management ingress"
		direction = "INGRESS"
	  },
	  {
		protocol = "6" # TCP
		source   = "0.0.0.0/0"
		destination   = "0.0.0.0/0"
		min_dstport   = 80
		min_srcport	= 80
		max_dstport   = 80
		max_srcport	= 80
		description = "management egress"
		direction = "EGRESS"
	  }
	]
  }

// -------------------
// zta config Info
// -------------------

variable "init_config" {
	description = "nZTA config downloaded from the controller"
	type        = string
	default     = "bas64config"
}


// -------------------
// Load balancer config Info
// -------------------

variable "lb_display_name" {
	description = "Display name of the loadbalancer"
	type        = string
	default     = "my_lb_name"
}

variable "listener_display_name" {
	description = "Display name of the listener"
	type        = list(string)
	default     = ["my_listener_80","my_listener_443"]
}

variable "bs_display_name" {
	description = "Display name of the backend set"
	type        = list(string)
	default     = ["my_back_end_set_80","my_back_end_set_443"]
}

variable "external_lb_id" {
	description = "Load balancer ID"
	type        = string
	default     = "my_lb_id"
}

variable "ports" {
	description = "Ports to set up listeners and backends for"
	type        = list(number)
	default     = [80, 443]
  }

  variable "protocol" {
	description = "Protocol for the backend set"
	type        =  string
	default     =  "TCP"
  }


  variable "bs_policy" {
	description = "Policy for the backend set"
	type        =  string
	default     =  "FIVE_TUPLE"
  }
